@@ -2402,6 +2402,31 @@ public class ASTParserLoadingTest extends FunctionalTestCase {
 
 		destroyTestBaseData();
 	}
+	public void testUnaryMinus(){
+        Session s = openSession();
+        s.beginTransaction();
+        Human stliu = new Human();
+        stliu.setIntValue( 26 );
+
+        s.persist( stliu );
+        s.getTransaction().commit();
+        s.clear();
+        s.beginTransaction();
+        List list =s.createQuery( "from Human h where -(h.intValue - 100)=74" ).list();
+        assertEquals( 1, list.size() );
+        s.getTransaction().commit();
+        s.clear();
+
+        s.beginTransaction();
+        s.createQuery( "delete java.lang.Object" ).executeUpdate();
+        s.getTransaction().commit();
+        s.close();
+
+
+
+
+    }
+    
 
 	public void testResultTransformerEntityQueries() throws Exception {
 
